from datetime import datetime
from typing import Generic, TypeVar, Optional

from mag_tools.exception.app_exception import AppException
from mag_tools.model.service_status import ServiceStatus

T = TypeVar('T')

class Result(Generic[T]):
    def __init__(self, status: Optional[ServiceStatus] = ServiceStatus.OK, code: Optional[ServiceStatus] = ServiceStatus.OK,
                 message: Optional[str] = None, data: Optional[T] = None):
        self.status = (status if status else ServiceStatus.OK).code
        self.code = (code if code else ServiceStatus.OK).code
        self.message = message if message else status.desc if status else None
        self.timestamp = datetime.now()
        self.data = data

    @staticmethod
    def exception(ex: Exception) -> 'Result':
        message = str(ex) if ex.args else str(ex.__cause__)
        return Result(status=ServiceStatus.INTERNAL_SERVER_ERROR, code=ServiceStatus.INTERNAL_SERVER_ERROR, message=message)

    @staticmethod
    def success(data: Optional[T] = None) -> 'Result':
        return Result(message="OK", data=data)

    @staticmethod
    def fail(message: str) -> 'Result':
        return Result(code=ServiceStatus.INTERNAL_SERVER_ERROR, message=message)

    @staticmethod
    def unauthorized(message: str) -> 'Result':
        return Result(status=ServiceStatus.UNAUTHORIZED, code=ServiceStatus.UNAUTHORIZED, message=message)

    @staticmethod
    def forbidden(message: str) -> 'Result':
        return Result(status=ServiceStatus.FORBIDDEN, code=ServiceStatus.FORBIDDEN, message=message)

    def is_success(self) -> bool:
        return self.status == ServiceStatus.OK and self.code == ServiceStatus.OK

    def check(self) -> None:
        if not self.is_success():
            raise AppException(self.message)

    def data(self) -> Optional[T]:
        return self.data


from typing import Optional
import usb.core
import usb.util

class UsbDevice:
    def __init__(self, vendor_id: Optional[int or str], product_id: Optional[int or str], device_id: Optional[str]=None,
                 caption: Optional[str] = None, system_name: Optional[str] = None, status: Optional[str] = None, description: Optional[str] = None) -> None:
        self.vendor_id = int(vendor_id, 16) if isinstance(vendor_id, str) else vendor_id
        self.product_id = int(product_id, 16) if isinstance(product_id, str) else product_id
        self.device_id = device_id
        self.caption = caption
        self.system_name = system_name
        self.status = status
        self.description = description
        self.manufacturer = None
        self.product = None
        self.serial_number = None
        self.__device = None

    def initialize(self):
        self.__device = self.__find(self.vendor_id, self.product_id, self.device_id)
        self.manufacturer = usb.util.get_string(self.__device, self.__device.iManufacturer)
        self.product = usb.util.get_string(self.__device, self.__device.iProduct)
        self.serial_number = usb.util.get_string(self.__device, self.__device.iSerialNumber)

    def read_data(self, endpoint, size):
        try:
            return self.__device.read(endpoint, size)
        except usb.core.USBError as e:
            raise f"读取数据失败: {e}"

    def write_data(self, endpoint, data):
        try:
            self.__device.write(endpoint, data)
        except usb.core.USBError as e:
            raise f"写入数据失败: {e}"

    def __str__(self):
        return (f"UsbDevice(VendorID={hex(self.vendor_id) if self.vendor_id else None}, "
                f"ProductID={hex(self.product_id) if self.product_id else None}, DeviceID={self.device_id}, "
                f"Caption={self.caption}, SystemName={self.system_name}, Status={self.status}, Description={self.description}, "
                f"Manufacturer={self.manufacturer}, Product={self.product}, SerialNumber={self.serial_number})")

    @classmethod
    def __find(cls, vendor_id: Optional[int] = None, product_id: Optional[int] = None, device_id: Optional[str] = None,):
        device = usb.core.find(idVendor= vendor_id, idProduct = product_id)
        if device:
            if device_id is None or device.serial_number == device_id:
                device.set_configuration()
                return device

        raise ValueError(f"[{vendor_id}.{product_id}]设备未找到")

# 示例使用
if __name__ == "__main__":
    _vendor_id = 0x781  #0x0529
    _product_id = 0x55ab    #0x1

    # 创建 UsbDevice 实例
    usb_device = UsbDevice(_vendor_id, _product_id)
    usb_device.initialize()
    print("设备信息:", usb_device)

    # 打印设备信息
    print(usb_device)

    # 查找设备
    _device = usb.core.find(idVendor=_vendor_id, idProduct=_product_id)
    if _device is None:
        raise ValueError("设备未找到")

    # 读取数据
    endpoint_in = 0x81  # 替换为你的设备的输入端点地址
    _data = UsbDevice.read_data(_device, endpoint_in, 64)
    if _data:
        print("读取的数据:", _data)

    # 写入数据
    endpoint_out = 0x01  # 替换为你的设备的输出端点地址
    UsbDevice.write_data(_device, endpoint_out, b"Hello USB")


from typing import Any, Optional

from mag_tools.bean.data_format import DataFormat
from mag_tools.model.justify_type import JustifyType


class TextFormat(DataFormat):
    def __init__(self, number_per_line: Optional[int] = 1, justify_type: Optional[JustifyType] = None,
                 at_header: Optional[str] = '', decimal_places: Optional[int] = 2,
                 decimal_places_of_zero: Optional[int] = 1, pad_length: Optional[int] = None, scientific: bool = False):
        """
        数据格式
        :param number_per_line: 每行显示的数据个数
        :param justify_type: 对齐方式
        :param at_header: 句首添加的字符串
        :param decimal_places: 小数位数
        :param pad_length: 字段显示长度，为 None 表示各字段自行定义
        :param decimal_places_of_zero: 小数为0时的小数位数
        """
        super().__init__(justify_type, decimal_places, decimal_places_of_zero, pad_length, scientific)

        self.number_per_line = number_per_line
        self.at_header = at_header
        self.scientific = scientific

    def __str__(self):
        """
        返回 TextFormat 实例的字符串表示。
        :return: TextFormat 实例的字符串表示。 """
        return f"TextFormat({', '.join(f'{key}={value}' for key, value in vars(self).items())})"

    def get_data_format(self, pad_length: int) -> DataFormat:
        return DataFormat(self.justify_type, self.decimal_places, self.decimal_places_of_zero, pad_length,
                          self.scientific)

    def get_data_format_by_value(self, value: Any):
        return self.get_data_format(len(str(value)))



import os
import json
from typing import Any, Dict, Optional


class SysConfig:
    __instance = None

    def __init__(self, root_dir: Optional[str]):
        # 获取工程的根目录
        self.__root_dir = root_dir if root_dir else os.getcwd()

        # 读取配置文件
        self.config = self.__load_config()

    @classmethod
    def set_root_dir(cls, root_dir):
        cls.__instance = None
        cls.__get_instance(root_dir).__root_dir = root_dir

    @classmethod
    def root_dir(cls):
        return cls.__get_instance().__root_dir

    @classmethod
    def resource_dir(cls):
        return os.path.join(cls.__get_instance().__root_dir, 'resources')

    @classmethod
    def logging_conf(cls):
        return os.path.join(cls.__get_instance().__root_dir, 'resources', 'config', 'logging.conf')

    @classmethod
    def get(cls, key: str, default=None):
        return cls.__get_instance().config.get(key, default)

    @classmethod
    def get_datasources(cls) -> Dict[str, Dict[str, Any]]:
        return cls.__get_instance().config.get("datasources")

    @classmethod
    def get_datasource(cls, datasource_name : str) -> Dict[str, Any]:
        datasources = cls.__get_instance().config.get("datasources")
        return datasources.get(datasource_name, None)

    @classmethod
    def __get_instance(cls, root_dir: Optional[str] = None):
        if not cls.__instance:
            cls.__instance = cls(root_dir)
        return cls.__instance

    def __load_config(self):
        config_file = os.path.join(self.__root_dir, 'resources', 'config', 'sys_config.json')

        if not os.path.exists(config_file):
            raise FileNotFoundError(f"配置文件 {config_file} 不存在")

        with open(config_file, 'r', encoding='utf-8') as file:
            config = json.load(file)
        return config



from datetime import datetime
from typing import Generic, TypeVar, Optional, List

from mag_tools.exception.app_exception import AppException
from mag_tools.model.service_status import ServiceStatus

T = TypeVar('T')

class Results(Generic[T]):
    def __init__(self, status: Optional[ServiceStatus] = ServiceStatus.OK, code: Optional[ServiceStatus] = ServiceStatus.OK,
                 message: Optional[str] = None, data: Optional[List[T]] = None):
        self.status = status.value if status else ServiceStatus.OK.value
        self.code = code.value if code else ServiceStatus.OK.value
        self.message = message if message else status.reason
        self.timestamp = datetime.now()
        self.data = data if data else []
        self.total_count = len(self.data)

    @staticmethod
    def exception(ex: Exception) -> 'Results':
        message = str(ex) if ex.args else str(ex.__cause__)
        return Results(status=ServiceStatus.INTERNAL_SERVER_ERROR, code=ServiceStatus.INTERNAL_SERVER_ERROR, message=message)

    @staticmethod
    def success(data: Optional[List[T]] = None) -> 'Results':
        return Results(message="OK", data=data)

    @staticmethod
    def fail(message: str) -> 'Results':
        return Results(code=ServiceStatus.INTERNAL_SERVER_ERROR, message=message)

    @staticmethod
    def unauthorized(message: str) -> 'Results':
        return Results(status=ServiceStatus.UNAUTHORIZED, code=ServiceStatus.UNAUTHORIZED, message=message)

    @staticmethod
    def forbidden(message: str) -> 'Results':
        return Results(status=ServiceStatus.FORBIDDEN, code=ServiceStatus.FORBIDDEN, message=message)

    def is_success(self) -> bool:
        return self.status == ServiceStatus.OK.value and self.code == ServiceStatus.OK.value

    def check(self) -> None:
        if not self.is_success():
            raise AppException(self.message)

    def size(self) -> int:
        return len(self.data)

    def get(self, idx: int) -> Optional[T]:
        self.check()
        return self.data[idx] if idx < self.size() else None

    def data(self) -> List[T]:
        return self.data


from datetime import datetime
from typing import List, Optional

from mag_tools.log.logger import Logger
from mag_tools.model.log_type import LogType
from mag_tools.utils.common.check_node import CheckNode


class TimeProbe:
    def __init__(self, name: str):
        """
        初始化 TimeProbe 类的实例
        :param name: 探测器标题
        """
        self.__name = name
        self.__start_time = int(datetime.now().timestamp()) * 1000  # 毫秒
        self.__previous_time = self.__start_time
        self.__check_nodes = [CheckNode(0, "初始节点", datetime.now(), 0, 0)]

    @staticmethod
    def get_probe(name: str = "性能监测"):
        """
        启动时间探测
        :param name: 探测器标题
        :return: TimeProbe 实例
        """
        return TimeProbe(name)

    def check(self, node_name: Optional[str] = None):
        """
        检测上次到当前检测点的时间差
        :param node_name: 节点名称
        :return: 当前节点信息
        """
        now_time = int(datetime.now().timestamp()) * 1000
        node = CheckNode(len(self.__check_nodes), node_name, datetime.now(), now_time - self.__previous_time, now_time - self.__start_time)
        self.__check_nodes.append(node)
        self.__previous_time = now_time
        return node

    def total_time(self):
        """
        检测开始到这次检测点的时间差
        :return: 时间差，单位：毫秒
        """
        return int(datetime.now().timestamp()) * 1000 - self.__start_time

    def print(self, *args):
        """
        打印节点信息
        :param args: 格式化参数
        :return: 当前节点信息
        """
        title = self.__name.format(*args)
        node = self.check(title)
        Logger.info(LogType.PERFORMANCE, f"[{title}]:\n  {node.log()}")
        return node

    def write_log(self):
        """
        写入日志信息
        """
        message = ''
        for node in self.__check_nodes:
            message += f"[node]:\n{node.log()}\n"

        Logger.info(LogType.PERFORMANCE, f"{self.__name}\n{message}")

    def get_check_nodes(self) -> List[CheckNode]:
        """
        获取所有检测节点
        :return: 检测节点列表
        """
        return self.__check_nodes



import re
import wmi
from mag_tools.bean.usb_device import UsbDevice
from mag_tools.model.usb_type import UsbType

class UsbTools:
    @staticmethod
    def list_usb(usb_type: UsbType = None):
        c = wmi.WMI()
        usb_devices = c.Win32_USBHub()
        device_list = []
        for device in usb_devices:
            match = re.search(r'VID[_]?([0-9A-F]+).*PID[_]?([0-9A-F]+)', device.PNPDeviceID, re.I)
            if not match:
                match = re.search(r'USB\\(?:ROOT_HUB30|HASP)\\([0-9A-F]+)&([0-9A-F]+)', device.PNPDeviceID, re.I)

            vendor_id = match.group(1) if match else None
            product_id = match.group(2) if match else None

            device_info = UsbDevice(vendor_id, product_id, device.DeviceID, device.Caption, device.SystemName, device.Status, device.Description)
            if usb_type is None or (usb_type.desc in device_info.description or usb_type.code in device_info.description):
                device_list.append(device_info)
        return device_list


from typing import Any, Dict, Type, TypeVar, Optional
from datetime import datetime, date
from decimal import Decimal
import json

K = TypeVar('K')
V = TypeVar('V')


class EasyMap(Dict[K, V]):
    """
    Map简化操作类

    @author xlcao
    @version 2.2
    @copyright Copyright (c) 2015
    """

    def __init__(self):
        super().__init__()

    @staticmethod
    def builder() -> 'EasyMap[str, Any]':
        return EasyMap()

    def add(self, key: K, value: V) -> 'EasyMap[K, V]':
        """
        添加键值及对应的数值

        :param key: 键值
        :param value: 数值
        :return: self
        """
        if value is not None:
            self[key] = value
        return self

    def get_string(self, key: Any) -> Optional[str]:
        """
        从Map中取字符串

        :param key: 键值
        :return: 字符串
        """
        return str(self.get(key)) if self.get(key) is not None else None

    def get_byte(self, key: Any) -> Optional[bytes]:
        """
        从Map中取字节

        :param key: 键值
        :return: 字节
        """
        value = self.get_string(key)
        return bytes(value, 'utf-8') if value else None

    def get_integer(self, key: Any) -> Optional[int]:
        """
        从Map中取整数

        :param key: 键值
        :return: 整数
        """
        value = self.get_string(key)
        try:
            return int(value) if value else None
        except ValueError:
            return None

    def get_short(self, key: Any) -> Optional[int]:
        """
        从Map中取短整数

        :param key: 键值
        :return: 短整数
        """
        return self.get_integer(key)

    def get_long(self, key: Any) -> Optional[int]:
        """
        从Map中取长整数

        :param key: 键值
        :return: 长整数
        """
        return self.get_integer(key)

    def get_double(self, key: Any) -> Optional[float]:
        """
        从Map中取Double

        :param key: 键值
        :return: Double
        """
        value = self.get_string(key)
        try:
            return float(value) if value else None
        except ValueError:
            return None

    def get_float(self, key: Any) -> Optional[float]:
        """
        从Map中取Float

        :param key: 键值
        :return: Float
        """
        return self.get_double(key)

    def get_big_decimal(self, key: Any) -> Optional[Decimal]:
        """
        从Map中取BigDecimal

        :param key: 键值
        :return: BigDecimal
        """
        value = self.get_string(key)
        try:
            return Decimal(value) if value else None
        except ValueError:
            return None

    def get_date(self, key: Any) -> Optional[date]:
        """
        从Map中取日期

        :param key: 键值
        :return: 日期
        """
        value = self.get_string(key)
        try:
            return datetime.strptime(value, '%Y-%m-%d').date() if value else None
        except ValueError:
            return None

    def get_local_date_time(self, key: Any) -> Optional[datetime]:
        """
        从Map中取日期时间

        :param key: 键值
        :return: 日期时间
        """
        value = self.get_string(key)
        try:
            return datetime.strptime(value, '%Y-%m-%d %H:%M:%S') if value else None
        except ValueError:
            return None

    def get_local_date(self, key: Any) -> Optional[date]:
        """
        从Map中取日期

        :param key: 键值
        :return: 日期
        """
        return self.get_date(key)

    def get_local_time(self, key: Any) -> Optional[datetime.time]:
        """
        从Map中取时间

        :param key: 键值
        :return: 时间
        """
        value = self.get_string(key)
        try:
            return datetime.strptime(value, '%H:%M:%S').time() if value else None
        except ValueError:
            return None

    def get_timestamp(self, key: str) -> Optional[datetime]:
        """
        从Map中取时间戳

        :param key: 键值
        :return: 时间戳
        """
        return self.get_local_date_time(key)

    def get_boolean(self, key: Any) -> bool:
        """
        从Map中取布尔值

        :param key: 键值
        :return: 布尔值
        """
        value = self.get_string(key)
        return value.lower() in ('y', 'yes', 't', 'true', '1') if value else False

    def get_enum(self, key: Any, enum_type: Type[V]) -> Optional[V]:
        """
        从Map中取枚举

        :param key: 键值
        :param enum_type: 枚举类型
        :return: 枚举值
        """
        value = self.get_string(key)
        if value and enum_type:
            try:
                return enum_type[value]
            except KeyError:
                return None
        return None

    def get_string_by_json(self, key: Any) -> Optional[str]:
        """
        从Map中取字符串（JSON）

        :param key: 键值
        :return: 字符串
        """
        return json.dumps(self.get(key))

    def get_byte_by_json(self, key: Any) -> Optional[bytes]:
        """
        从Map中取字节（JSON）

        :param key: 键值
        :return: 字节
        """
        value = self.get_string_by_json(key)
        return bytes(value, 'utf-8') if value else None

    def get_short_by_json(self, key: Any) -> Optional[int]:
        """
        从Map中取短整数（JSON）

        :param key: 键值
        :return: 短整数
        """
        return self.get_integer_by_json(key)

    def get_integer_by_json(self, key: Any) -> Optional[int]:
        """
        从Map中取整数（JSON）

        :param key: 键值
        :return: 整数
        """
        value = self.get_string_by_json(key)
        try:
            return int(value) if value else None
        except ValueError:
            return None

    def get_long_by_json(self, key: Any) -> Optional[int]:
        """
        从Map中取长整数（JSON）

        :param key: 键值
        :return: 长整数
        """
        return self.get_integer_by_json(key)

    def get_double_by_json(self, key: Any) -> Optional[float]:
        """
        从Map中取Double（JSON）

        :param key: 键值
        :return: Double
        """
        value = self.get_string_by_json(key)
        try:
            return float(value) if value else None
        except ValueError:
            return None

    def get_float_by_json(self, key: Any) -> Optional[float]:
        """
        从Map中取Float（JSON）

        :param key: 键值
        :return: Float
        """
        return self.get_double_by_json(key)

    def get_big_decimal_by_json(self, key: Any) -> Optional[Decimal]:
        """
        从Map中取BigDecimal（JSON）

        :param key: 键值
        :return: BigDecimal
        """
        value = self.get_string_by_json(key)
        try:
            return Decimal(value) if value else None
        except ValueError:
            return None

    def get_date_by_json(self, key: Any) -> Optional[date]:
        """
        从Map中取日期（JSON）

        :param key: 键值
        :return: 日期
        """
        value = self.get_string_by_json(key)
        try:
            return datetime.strptime(value, '%Y-%m-%d').date() if value else None
        except ValueError:
            return None

    def get_local_date_time_by_json(self, key: Any) -> Optional[datetime]:
        """
        从Map中取日期时间（JSON）

        :param key: 键值
        :return: 日期时间
        """
        value = self.get_string_by_json(key)
        try:
            return datetime.strptime(value, '%Y-%m-%d %H:%M:%S') if value else None
        except ValueError:
            return None

    def get_local_date_by_json(self, key: Any) -> Optional[date]:
        """
        从Map中取日期（JSON）

        :param key: 键值
        :return: 日期
        """
        return self.get_date_by_json(key)

    def get_local_time_by_json(self, key: Any) -> Optional[datetime.time]:
        """
        从Map中取时间（JSON）

        :param key: 键值
        :return: 时间
        """
        value = self.get_string_by_json(key)
        try:
            return datetime.strptime(value, '%H:%M:%S').time() if value else None
        except ValueError:
            return None

    def get_timestamp_by_json(self, key: str) -> Optional[datetime]:
        """
        从Map中取时间戳（JSON）

        :param key: 键值
        :return: 时间戳
        """
        return self.get_local_date_time_by_json(key)

    def get_boolean_by_json(self, key: Any) -> bool:
        """
        从Map中取布尔值（JSON）

        :param key: 键值
        :return: 布尔值
        """
        value = self.get_string_by_json(key)
        return value.lower() in ('y', 'yes', 't', 'true', '1') if value else False

    def get_enum_by_json(self, key: Any, enum_type: Type[V]) -> Optional[V]:
        """
        从Map中取枚举（JSON）

        :param key: 键值
        :param enum_type: 枚举类型
        :return: 枚举值
        """
        value = self.get_string_by_json(key)
        if value and enum_type:
            try:
                return enum_type[value]
            except KeyError:
                return None
        return None

    def put(self, key: K, value: V) -> V:
        """
        添加键值及对应的数值

        :param key: 键值
        :param value: 数值
        :return: 数值
        """
        return super().__setitem__(key, value)

    def to_map(self) -> Dict[K, V]:
        """
        转为Map

        :return: Map
        """
        return dict(self)

    def get_by_json(self, key: Any, clazz: Type[V]) -> Optional[V]:
        """
        从Map中取对象（JSON）

        :param key: 键值
        :param clazz: 类类型
        :return: 对象
        """
        value = self.get(key)
        return json.loads(value, object_hook=clazz) if value else None
